@@ -3,15 +3,15 @@ import urwid
 
 from concurrent.futures import ThreadPoolExecutor
 
-from toot import api, __version__
+from toot import api, config, __version__
 
 from .compose import StatusComposer
 from .constants import PALETTE
 from .entities import Status
-from .overlays import ExceptionStackTrace, GotoMenu, Help, StatusSource
+from .overlays import ExceptionStackTrace, GotoMenu, Help, StatusSource, StatusLinks
 from .overlays import StatusDeleteConfirmation
 from .timeline import Timeline
-from .utils import show_media
+from .utils import parse_content_links, show_media
 
 logger = logging.getLogger(__name__)
 
@@ -88,6 +88,7 @@ class TUI(urwid.Frame):
     def __init__(self, app, user):
         self.app = app
         self.user = user
+        self.config = config.load_config()
 
         self.loop = None  # set in `create`
         self.executor = ThreadPoolExecutor(max_workers=1)
@@ -182,6 +183,9 @@ class TUI(urwid.Frame):
         def _source(timeline, status):
             self.show_status_source(status)
 
+        def _links(timeline, status):
+            self.show_links(status)
+
         def _media(timeline, status):
             self.show_media(status)
 
@@ -197,8 +201,9 @@ class TUI(urwid.Frame):
         urwid.connect_signal(timeline, "reblog", self.async_toggle_reblog)
         urwid.connect_signal(timeline, "reply", _reply)
         urwid.connect_signal(timeline, "source", _source)
+        urwid.connect_signal(timeline, "links", _links)
 
-    def build_timeline(self, name, statuses):
+    def build_timeline(self, name, statuses, local):
         def _close(*args):
             raise urwid.ExitMainLoop()
 
@@ -208,12 +213,28 @@ class TUI(urwid.Frame):
         def _thread(timeline, status):
             self.show_thread(status)
 
+        def _toggle_save(timeline, status):
+            if not timeline.name.startswith("#"):
+                return
+            hashtag = timeline.name[1:]
+            assert isinstance(local, bool), local
+            timelines = self.config.setdefault("timelines", {})
+            if hashtag in timelines:
+                del timelines[hashtag]
+                self.footer.set_message("#{} unpinned".format(hashtag))
+            else:
+                timelines[hashtag] = {"local": local}
+                self.footer.set_message("#{} pinned".format(hashtag))
+            self.loop.set_alarm_in(5, lambda *args: self.footer.clear_message())
+            config.save_config(self.config)
+
         timeline = Timeline(name, statuses)
 
         self.connect_default_timeline_signals(timeline)
         urwid.connect_signal(timeline, "next", _next)
         urwid.connect_signal(timeline, "close", _close)
         urwid.connect_signal(timeline, "thread", _thread)
+        urwid.connect_signal(timeline, "save", _toggle_save)
 
         return timeline
 
@@ -244,7 +265,7 @@ class TUI(urwid.Frame):
         self.body = timeline
         self.refresh_footer(timeline)
 
-    def async_load_timeline(self, is_initial, timeline_name=None):
+    def async_load_timeline(self, is_initial, timeline_name=None, local=None):
         """Asynchronously load a list of statuses."""
 
         def _load_statuses():
@@ -260,7 +281,7 @@ class TUI(urwid.Frame):
 
         def _done_initial(statuses):
             """Process initial batch of statuses, construct a Timeline."""
-            self.timeline = self.build_timeline(timeline_name, statuses)
+            self.timeline = self.build_timeline(timeline_name, statuses, local)
             self.timeline.refresh_status_details()  # Draw first status
             self.refresh_footer(self.timeline)
             self.body = self.timeline
@@ -293,8 +314,8 @@ class TUI(urwid.Frame):
         status, index, count = timeline.get_focused_status_with_counts()
         self.footer.set_status([
             ("footer_status_bold", "[{}] ".format(timeline.name)),
-            status.id, " - status ", str(index + 1), " of ", str(count),
-        ])
+        ] + ([status.id, " - status ", str(index + 1), " of ", str(count)]
+            if status else ["no focused status"]))
 
     def show_status_source(self, status):
         self.open_overlay(
@@ -302,6 +323,15 @@ class TUI(urwid.Frame):
             title="Status source",
         )
 
+    def show_links(self, status):
+        links = parse_content_links(status.data["content"]) if status else []
+        if links:
+            self.open_overlay(
+                widget=StatusLinks(links),
+                title="Status links",
+                options={"height": len(links) + 2},
+            )
+
     def show_exception(self, exception):
         self.open_overlay(
             widget=ExceptionStackTrace(exception),
@@ -321,7 +351,8 @@ class TUI(urwid.Frame):
         self.open_overlay(composer, title="Compose status")
 
     def show_goto_menu(self):
-        menu = GotoMenu()
+        user_timelines = self.config.get("timelines", {})
+        menu = GotoMenu(user_timelines)
         urwid.connect_signal(menu, "home_timeline",
             lambda x: self.goto_home_timeline())
         urwid.connect_signal(menu, "public_timeline",
@@ -331,7 +362,7 @@ class TUI(urwid.Frame):
 
         self.open_overlay(menu, title="Go to", options=dict(
             align="center", width=("relative", 60),
-            valign="middle", height=9,
+            valign="middle", height=9 + len(user_timelines),
         ))
 
     def show_help(self):
@@ -345,14 +376,16 @@ class TUI(urwid.Frame):
 
     def goto_public_timeline(self, local):
         self.timeline_generator = api.public_timeline_generator(
-            self.app.instance, local=local, limit=40)
+            self.app, self.user, local=local, limit=40)
         promise = self.async_load_timeline(is_initial=True, timeline_name="public")
         promise.add_done_callback(lambda *args: self.close_overlay())
 
     def goto_tag_timeline(self, tag, local):
         self.timeline_generator = api.tag_timeline_generator(
-            self.app.instance, tag, local=local, limit=40)
-        promise = self.async_load_timeline(is_initial=True, timeline_name="#{}".format(tag))
+            self.app, self.user, tag, local=local, limit=40)
+        promise = self.async_load_timeline(
+            is_initial=True, timeline_name="#{}".format(tag), local=local,
+        )
         promise.add_done_callback(lambda *args: self.close_overlay())
 
     def show_media(self, status):
@@ -489,6 +522,11 @@ class TUI(urwid.Frame):
         elif key == 'esc':
             if self.overlay:
                 self.close_overlay()
+            elif self.timeline.name != "home":
+                # similar to goto_home_timeline() but without handling overlay (absent here)
+                self.timeline_generator = api.home_timeline_generator(
+                    self.app, self.user, limit=40)
+                self.async_load_timeline(is_initial=True, timeline_name="home")
 
         elif key in ('q', 'Q'):
             if self.overlay:
