@@ -20,6 +20,20 @@ class StatusSource(urwid.ListBox):
         super().__init__(walker)
 
 
+class StatusLinks(urwid.ListBox):
+    """Shows status links."""
+
+    def __init__(self, links):
+
+        def widget(url, title):
+            return Button(title or url, on_press=lambda btn: webbrowser.open(url))
+
+        walker = urwid.SimpleFocusListWalker(
+            [widget(url, title) for url, title in links]
+        )
+        super().__init__(walker)
+
+
 class ExceptionStackTrace(urwid.ListBox):
     """Shows an exception stack trace."""
     def __init__(self, ex):
@@ -54,17 +68,17 @@ class GotoMenu(urwid.ListBox):
         "hashtag_timeline",
     ]
 
-    def __init__(self):
+    def __init__(self, user_timelines):
         self.hash_edit = EditBox(caption="Hashtag: ")
 
-        actions = list(self.generate_actions())
+        actions = list(self.generate_actions(user_timelines))
         walker = urwid.SimpleFocusListWalker(actions)
         super().__init__(walker)
 
     def get_hashtag(self):
         return self.hash_edit.edit_text.strip()
 
-    def generate_actions(self):
+    def generate_actions(self, user_timelines):
         def _home(button):
             self._emit("home_timeline")
 
@@ -81,7 +95,18 @@ class GotoMenu(urwid.ListBox):
             else:
                 self.set_focus(4)
 
+        def mk_on_press_user_hashtag(tag, local):
+            def on_press(btn):
+                self._emit("hashtag_timeline", tag, local)
+            return on_press
+
         yield Button("Home timeline", on_press=_home)
+
+        for tag, cfg in user_timelines.items():
+            is_local = cfg["local"]
+            yield Button("#{}".format(tag) + (" (local)" if is_local else ""),
+                         on_press=mk_on_press_user_hashtag(tag, is_local))
+
         yield Button("Local public timeline", on_press=_local_public)
         yield Button("Global public timeline", on_press=_global_public)
         yield urwid.Divider()
@@ -120,6 +145,7 @@ class Help(urwid.Padding):
         yield urwid.Divider()
         yield urwid.Text(h("  [Q] - quit toot"))
         yield urwid.Text(h("  [G] - go to - switch timelines"))
+        yield urwid.Text(h("  [P] - save/unsave (pin) current timeline"))
         yield urwid.Text(h("  [H] - show this help"))
         yield urwid.Divider()
         yield urwid.Text(("bold", "Status keys"))
@@ -132,6 +158,7 @@ class Help(urwid.Padding):
         yield urwid.Text(h("  [R] - Reply to current status"))
         yield urwid.Text(h("  [S] - Show text marked as sensitive"))
         yield urwid.Text(h("  [T] - Show status thread (replies)"))
+        yield urwid.Text(h("  [L] - Show the status links"))
         yield urwid.Text(h("  [U] - Show the status data in JSON as received from the server"))
         yield urwid.Text(h("  [V] - Open status in default browser"))
         yield urwid.Divider()
